#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

import sys
sys.path.append("..")
sys.path.append(".")

import os
print(os.getcwd())
#sys.path.append("C:/Users/natal/python_driver1")

from keysql_client.keysql_connection import KeySqlConnection
from keysql_client.client import KeySqlClient

from keysql_client.key_object import KeyObjectNumber, KeyObjectChar, KeyObjectComposition, KeyObjectMultiple
from keysql_client.keysql_catalog import KeySqlCatalog
from keysql_client.keysql_catalog_statement import *
from keysql_client.keysql_store_statement import *
from keysql_client.key_object_statement import *
from keysql_client.keysql_parser import *

import logging
import argparse

"""
Example:
create catalog birds;
create store home catalog  backyard_birds;

create keyobject season char in catalog birds;
create keyobject act_type char in catalog birds;
create keyobject activity {act_type, season} in catalog birds;
create keyobject name char in catalog birds;
create keyobject bird {name, activity} in catalog birds;

insert into backyard_birds instances
{bird: {name: 'house sparrow', activity: {act_type: 'bath',  season: 'summer'}}},
{bird: {name: 'house sparrow', activity: {act_type: 'eat',  season: 'summer'}}},
{bird: {name: 'house sparrow', activity: {act_type: 'sing',  season: 'summer'}}},
{bird: {name: 'house sparrow', activity: {act_type: 'bath',  season: 'fall'}}},
{bird: {name: 'house sparrow', activity: {act_type: 'eat',  season: 'fall'}}},
{bird: {name: 'house sparrow', activity: {act_type: 'sing',  season: 'fall'}}},
{bird: {name: 'white-crowned sparrow', activity: {act_type: 'bath',  season: 'fall'}}},
{bird: {name: 'white-crowned sparrow', activity: {act_type: 'eat',  season: 'fall'}}},
{bird: {name: 'white-crowned sparrow', activity: {act_type:  'sing',  season: 'fall'}}};
"""

def clean_catalog(test_connection, catalog_name):
    all_catalogs = test_connection.execute("list catalogs")

    # catalog_name does not exists => nothing to clean
    if catalog_name.upper() not in all_catalogs:
        return

    stores = test_connection.execute('list stores for catalog {}'.format(catalog_name))
    # if there are stores, remove them
    for store in stores:
        reply = test_connection.execute('drop store {}'.format(store))
    # drop catalog
    reply = test_connection.execute('drop catalog {}'.format(catalog_name))


def test_create_db_as_string_call(test_connection) -> None:

    # drop catalog if exists (comment if not needed)
    clean_catalog(test_connection, 'birds')

    test_connection.execute('create catalog birds')
    test_connection.execute('create store backyard_birds for catalog  birds')

    test_connection.execute('create keyobject season char in catalog birds')
    test_connection.execute('create keyobject act_type char in catalog birds')
    test_connection.execute('create keyobject name char in catalog birds')
    test_connection.execute('create keyobject activity {act_type, season} in catalog birds')
    test_connection.execute('create keyobject bird {name, activity} in catalog birds')

    reply = test_connection.execute("show catalog birds", keep_str_res=True)
    print('\nshow catalog birds ...................................')
    print('catalog birds: {}'.format(test_connection.get_last_str_result()))

    print('\nselect * from backyard_birds (nothing yet, empty string) ...................................')
    reply = test_connection.execute("select * from backyard_birds", keep_str_res=True)
    print(test_connection.get_last_str_result())

    reply = test_connection.execute("insert into backyard_birds instances {bird: {name: 'house sparrow', activity: {act_type: 'bath',  season: 'summer'}}}", keep_str_res=True)

    print('\nselect * from backyard_birds (one instances inserted ) ...................................')
    reply = test_connection.execute("select * from backyard_birds", keep_str_res=True)
    print(test_connection.get_last_str_result())

    test_connection.execute("insert into backyard_birds instances \
      {bird: {name: 'house sparrow', activity: {act_type: 'eat',  season: 'summer'}}},\
      {bird: {name: 'house sparrow', activity: {act_type: 'sing',  season: 'summer'}}},\
      {bird: {name: 'house sparrow', activity: {act_type: 'bath', season: 'fall'}}},\
      {bird: {name: 'house sparrow', activity: {act_type: 'eat', season: 'fall'}}},\
      {bird: {name: 'house sparrow', activity: {act_type: 'sing', season: 'fall'}}},\
      {bird: {name: 'white-crowned sparrow', activity: {act_type: 'bath', season: 'fall'}}},\
      {bird: {name: 'white-crowned sparrow', activity: {act_type: 'eat', season: 'fall'}}},\
      {bird: {name: 'white-crowned sparrow', activity: {act_type: 'sing', season: 'fall'}}}", keep_str_res=True)

    reply = test_connection.execute("select * from backyard_birds", keep_str_res=True)
    print('\nselect * from backyard_birds (several instances inserted ) ...................................')
    reply = test_connection.execute("select * from backyard_birds", keep_str_res=True)
    print(test_connection.get_last_str_result())

    print('\nselect name, season, count(season) as cnt from backyard_birds group by season ...................................')
    reply = test_connection.execute('create keyobject cnt number in catalog birds')
    reply = test_connection.execute("select name, season, count(season) as cnt from backyard_birds group by season", keep_str_res=True)
    print(test_connection.get_last_str_result())

    print('\nshow store backyard_birds ...................................')
    reply = test_connection.execute("show store backyard_birds", keep_str_res=True)
    print('store birds: {}'.format(test_connection.get_last_str_result()))



if __name__ == "__main__":
    #uncomment to see log
    #logging.basicConfig(level=logging.DEBUG)

    parser = argparse.ArgumentParser()

    # Host can be ip address or full_name, examples:  "10.0.0.123", "my_server_name.com"
    parser.add_argument('--host', required=False, default='ip_address_or_host_name.com', type=str)
    parser.add_argument('--port', required=False, default=1234, type=int)
    parser.add_argument('--user', required=True, type=str)
    parser.add_argument('--password', required=True, type=str)

    args = parser.parse_args()

    print('args', args)

    # instatiate client
    keysql = KeySqlClient(args.host, args.port)
    test_connection = None

    # connect with server
    try:
        test_connection = keysql.connect(args.user, args.password)
    except ValueError as ex:
        logging.error(ex)  # ex.message())

    if not test_connection:
         exit()

    #test_host, test_port = args.host, args.port
    #test_user_name, my_passport = args.user, args.password

    # connect with credentials
    #test_connection = KeySqlConnection(test_host, test_port, test_user_name, my_passport)

    # run queries
    test_create_db_as_string_call(test_connection)

    # disconnect at the end of the session
    test_connection.disconnect()
